Package: lme4breeding
Version: 1.1.2
Date: 2026-05-20
Title: Breeding-Related Mixed-Effects Models
Authors@R: c(
    person("Giovanny","Covarrubias-Pazaran", email="cova_ruber@live.com.mx", role=c("aut","cre"),
	comment=c(ORCID="0000-0002-7194-3837"))
	)
Maintainer: Giovanny Covarrubias-Pazaran <cova_ruber@live.com.mx>
Description: Fit relationship-based and customized mixed-effects models with complex variance-covariance structures using the 'lme4' machinery. The core computational algorithms are implemented using the
    'Eigen' 'C++' library for numerical linear algebra and 'RcppEigen' 'glue'.
Depends: R(>= 3.5.0), lme4 (>= 1.0), Matrix (>= 1.0), methods, crayon,
        enhancer
LazyLoad: yes
License: GPL (>= 2)
Author: Giovanny Covarrubias-Pazaran [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7194-3837>)
Repository: CRAN
Suggests: rmarkdown, knitr, orthopolynom, MASS, RSpectra
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-05-21 03:20:50 UTC; giovannycovarrubias
Date/Publication: 2026-05-21 02:20:02 UTC
Built: R 4.5.2; ; 2026-05-25 01:45:46 UTC; unix
