
import("methods")
import("Matrix")
import("lme4")
import("enhancer")
import("crayon")

importFrom("utils", "setTxtProgressBar", "txtProgressBar")

importFrom("stats", "as.formula", "cov", "vcov", "terms", "model.matrix", "na.omit",
           "na.pass", "median", "reshape","var","aggregate", "sigma")

export( "lmebreed", "lmeb", "smm",
        "getMME",
        "adjBeta", "umat", 
        "balanceData", "Dtable", "condVarRotated","mkMmeIndex",
        "predict.lmeb"
        )

exportMethods(
              "coerce",
              "fitted",
              "ranef",
              "residuals"
              )

S3method(predict, lmeb)

