# lme4breeding: Linear Mixed Equations 4 Breeding

Extension of the popular lme4 package from Bates et al. to easily fit the most popular 
breeding models heterogeneous and unstructured variance models, genomic models, 
competition models, overlayed models, etc.

## Installation

You can install the development version of `lme4breeding` from GitHub:

``` r
devtools::install_github('covaruber/lme4breeding')
```
In addition, the package is also available on CRAN:

``` r
install.packages("lme4breeding")
```

## Vignettes

The package is equiped with many datasets and vignettes to learn how to use the package. 